using OptimKit

#all input sanitizing is done on the matlab side
function mexed_optimize(args::Vector{MATLAB.MxArray})
    length(args) == 6 || throw(ArgumentError("wrong number of arguments"))

    fg_fun = args[1];
    startpoint = args[2];
    algorithm = _init_algorithm(jvalue(args[3]));

    # initially tried working with a struct of function handles, appears unsupported
    matscale = args[4];
    matretract = args[5];
    matinner = args[6];

    function j_fg_fun(x)
        (a,b) = call_matlab(2,"feval",[fg_fun,x]);
        real(jvalue(a)),b
    end

    j_inner(a,b,c) = jvalue(first(call_matlab(1,"feval",[matinner,a,b,c])))
    j_scale(a,b::Number) = first(call_matlab(1,"feval",[matscale,a,MATLAB.mxarray(b)]))
    j_retract(a,b,c::Number) = tuple(call_matlab(2,"feval",[matretract,a,b,MATLAB.mxarray(c)])...)

    (x, fx, gx, numfg, normgradhistory) = optimize(j_fg_fun,startpoint,algorithm,inner = j_inner,retract = j_retract, scale! = j_scale)
end

#parses the opts - dictrionary, and returns a proper julia algorithm structure
function _init_algorithm(opts)
    if opts["type"] == "GradientDescent"
        return GradientDescent();
    else
        throw(ArgumentError("unknown algorithm"))
    end
end
